#ifdef _WIN32
#include "stdafx.h"
#endif

#include <stdarg.h>		// Required for variable length arguments
#include <stdio.h>		// Required for file IO and NULL declaration
#include <stdlib.h>		// Required for memory manipulation
#include <string.h>
#include <time.h>
#include <BipCci.h>		// Common functions (between nodes and parsers)
#include <BipCni.h>		// Message processing nodes
#include "NodeUtils.h"

//
// Uncomment the following line to use wcs* functions.  This can only be
// done if the wchar_t is identical to the CciChar data type.  Currently
// true for NT and AIX but NOT true for Solaris.
//
//#define USE_WCHAR_T (1)

///////////////////////////////////////////////////////////
// mqsixCciCharCpy
// Copy the source string to the destination string using the
// CciChar data type.  Source string is expected to be terminated
// by a 0 value.
static void mqsixCciCharCpy(CciChar *destination, CciChar *source)
{
	if (destination == NULL || source == NULL)
	{
		return;
	}
	while(*source != 0)
	{
		*destination = *source;
		destination++;
		source++;
	}
	*destination = *source;
}

///////////////////////////////////////////////////////////
// mqsixCciCharLen
// Calculate the length of the string for CciChar data types.
static int mqsixCciCharLen(CciChar *string)
{
	int iLength = 0;
	if (string == NULL)
	{
		return(0);
	}
	while(*string != 0)
	{
		string++;
		iLength++;
	}
	return(iLength);
}

///////////////////////////////////////////////////////////
// mqsixCciCharCmp
// Compare two CciChar strings ...
static int mqsixCciCharCmp(CciChar *stringOne, CciChar *stringTwo)
{
	if (stringOne == NULL || stringTwo == NULL)
	{
		return(-1);
	}
	while(*stringOne != 0 && *stringTwo != 0)
	{
		if (*stringOne < *stringTwo)
		{
			return(-1);
		}
		if (*stringOne > *stringTwo)
		{
			return(1);
		}
		stringOne++;
		stringTwo++;
	}
	if (*stringOne != 0)
	{
		return(1);
	}
	if (*stringTwo != 0)
	{
		return(-1);
	}
	return(0);
}


///////////////////////////////////////////////////////////
// mqsixCopyString
// Copy the CciChar strings ensuring there is enough storage
int mqsixStringCopy(CciChar *source, CciChar *destination, unsigned int iLength)
{
	// Ensure that there is enough space to copy the strings
	if (mqsixStringLengthBytes(source) > iLength)
	{
		return(1);
	}
#ifdef USE_WCHAR_T
	wcscpy(destination, source);
#else
	mqsixCciCharCpy(destination, source);
#endif
	return(0);
}




///////////////////////////////////////////////////////////
// mqsixStringLengthCharacters
// Return the number of characters contained within the
// string.  Does not include any terminator character ('\0')
unsigned int mqsixStringLengthCharacters(CciChar *string)
{
	if (string == NULL)
	{
		return(0);
	}
#ifdef USE_WCHAR_T
	return(wcslen(string));
#else
	return(mqsixCciCharLen(string));
#endif
}

///////////////////////////////////////////////////////////
// mqsixStringLengthBytes
// Return the length in bytes of memory that the string
// occupies.  This includes the trailing \0 character
// terminator.
unsigned int mqsixStringLengthBytes(CciChar *string)
{
	if (string == NULL)
	{
		return(0);
	}
	return(sizeof(CciChar) * (mqsixStringLengthCharacters(string) + 1));
}

///////////////////////////////////////////////////////////
// mqsixStringCompare
int mqsixStringCompare(CciChar *stringOne, CciChar *stringTwo)
{
#ifdef USE_WCHAR_T
	return(wcscmp(stringOne, stringTwo));
#else
	return(mqsixCciCharCmp(stringOne, stringTwo));
#endif
}

///////////////////////////////////////////////////////////
// mqsixStringDuplicate
CciChar *mqsixStringDuplicate(CciChar *string)
{
	CciChar *temp;
	int		iSize;

	iSize = mqsixStringLengthBytes(string);
	temp = (CciChar *)malloc(iSize);
	mqsixStringCopy(string, temp, iSize);
	return(temp);
}

// **************
// * ATTRIBUTES *
// **************

///////////////////////////////////////////////////////////
// mqsixAddAttribute
void mqsixAddAttribute(MQSIX_ATTRIBUTES **pAttribute,
					   CciChar *strName,
					   CciChar *strValue)
{
	MQSIX_ATTRIBUTES *pTempAttribute;
	MQSIX_ATTRIBUTES *pTempAttribute2;

	pTempAttribute = (MQSIX_ATTRIBUTES *)malloc(sizeof(MQSIX_ATTRIBUTES));
	pTempAttribute->attributeName = mqsixStringDuplicate(strName);
	pTempAttribute->attributeValue = mqsixStringDuplicate(strValue);
	pTempAttribute->pNext = NULL;
	if ((*pAttribute) == NULL)
	{
		*pAttribute = pTempAttribute;
	}
	else
	{
		pTempAttribute2 = *pAttribute;
		while(pTempAttribute2->pNext != NULL)
		{
			pTempAttribute2 = pTempAttribute2->pNext;
		}
		pTempAttribute2->pNext = pTempAttribute;
	}
	return;
}

///////////////////////////////////////////////////////////
// mqsixReleaseAttributes
void mqsixReleaseAttributes(MQSIX_ATTRIBUTES *pAttributes)
{
	while(pAttributes != NULL)
	{
		free(pAttributes->attributeName);
		free(pAttributes->attributeValue);
		pAttributes = pAttributes->pNext;
	}
}

///////////////////////////////////////////////////////////
// mqsixGetAttributeName
int mqsixGetAttributeName(
	MQSIX_ATTRIBUTES	*pAttribute,
	int					iIndex,
	CciChar				*buffer,
	int					bufsize)
{
	int		iCount = 0;

	mqsixLog(">> mqsixGetAttributeName:  Index=%d\n", iIndex);
	if (pAttribute == NULL)
	{
		mqsixLog("--    No attribute supplied!!\n");
		return(1);
	}
	while(iIndex > 0)
	{
		pAttribute = pAttribute->pNext;
		if (pAttribute == NULL)
		{
			mqsixLog("--   Index too large, highest: %d\n", iCount);
			return(1);
		}
		iIndex--;
		iCount++;
	}
	mqsixStringCopy(pAttribute->attributeName, buffer, bufsize);
	mqsixLog("--   Returning name: %S\n", pAttribute->attributeName);
	return(0);
}

///////////////////////////////////////////////////////////
// mqsixGetAttribute
int mqsixGetAttribute(
	MQSIX_ATTRIBUTES	*pAttribute,
	CciChar				*attrName,
	CciChar				*buffer,
	int					bufsize)
{
	mqsixLog(">> mqsixGetAttribute: %S\n", attrName);
	if (pAttribute == NULL)
	{
		mqsixLog("--    No attribute supplied!!\n");
		return(1);
	}
	while(pAttribute != NULL)
	{
		if (mqsixStringCompare(attrName, pAttribute->attributeName) == 0)
		{
			if (pAttribute->attributeValue == NULL)
			{
				mqsixLog("--   Name found but no value\n");
				return(1);
			}
			mqsixStringCopy(pAttribute->attributeValue, buffer, bufsize);
			mqsixLog("--   Returning: \"%S\"\n", buffer);
			return(0);
		}
		pAttribute = pAttribute->pNext;
	}
	mqsixLog("--   No matching name found\n");
	return(1);
}

///////////////////////////////////////////////////////////
// mqsixSetAttribute
int mqsixSetAttribute(
	MQSIX_ATTRIBUTES	*pAttribute,
	CciChar				*attrName,
	CciChar				*attrValue)
{
	CciChar *temp;

	mqsixLog(">> mqsixSetAttribute: %S=\"%S\"\n", attrName, attrValue);
	if (pAttribute == NULL)
	{
		mqsixLog("--    No attribute supplied!!\n");
		return(1);
	}
	while(pAttribute != NULL)
	{
		if (mqsixStringCompare(attrName, pAttribute->attributeName) == 0)
		{
			temp = mqsixStringDuplicate(attrValue);
			if (pAttribute->attributeValue != NULL)
			{
				free(pAttribute->attributeValue);
			}
			pAttribute->attributeValue = temp;
			return(0);
		}
		pAttribute = pAttribute->pNext;
	}
	mqsixLog("No attribute named %S located\n", attrName);
	return(1);
}

///////////////////////////////////////////////////////////
// mqsixLog
#ifdef _WIN32
#define LOGFILE "C:\\NODE_MQGET.TXT"
#else
#define LOGFILE "/tmp/NODE_MQGET.TXT"
#endif

void mqsixLog(char *szFormat, ...)
{
#ifndef _DEBUG
	return;
#else
	static FILE	*logFile = NULL;
	struct tm	*now_tm;
	time_t		iNow;
	va_list		list;

	va_start(list, szFormat);

	if (logFile == NULL)
	{
		logFile = fopen(LOGFILE, "a");
		if (logFile == NULL)
		{
			return;
		}
	}

	iNow = time(NULL);
	now_tm = localtime(&iNow);
	fprintf(logFile, "%.2d/%.2d/%.4d %.2d:%.2d:%.2d ",
		now_tm->tm_mon,
		now_tm->tm_mday,
		now_tm->tm_year+1900,
		now_tm->tm_hour,
		now_tm->tm_min,
		now_tm->tm_sec);
	vfprintf(logFile, szFormat, list);
	fflush(logFile);
	va_end(list);
#endif
}

///////////////////////////////////////////////////////////
// mqsixElementCharValue
CciSize mqsixElementCharValue(CciElement *pElement, CciChar **buffer)
{
	int	iRc;
	CciChar	oneChar;
	CciChar	*pRetBuffer = NULL;
	CciSize	iSize;
	iSize = cniElementCharacterValue(&iRc, pElement, &oneChar, sizeof(oneChar));

	if (iRc == CCI_BUFFER_TOO_SMALL || iRc == 0)
	{
		pRetBuffer = (CciChar *)malloc(iSize * sizeof(CciChar));
		iSize = cniElementCharacterValue(&iRc, pElement, pRetBuffer, iSize);
		//pRetBuffer[iSize] = 0;
		if (iRc != 0)
		{
			free(pRetBuffer);
			pRetBuffer = NULL;
			iSize = 0;
		}
	}
	*buffer = pRetBuffer;
	return(iSize);
}

///////////////////////////////////////////////////////////
// mqsixElementByteArrayValue
CciSize mqsixElementByteArrayValue(CciElement *pElement, struct CciByteArray *buffer)
{
	int	iRc;
	struct CciByteArray oneByteArray;
	char	oneByte;
	CciChar	*pRetBuffer = NULL;
	CciSize	iSize;

	oneByteArray.pointer = &oneByte;
	oneByteArray.size = 1;
	iSize = cniElementByteArrayValue(&iRc, pElement, &oneByteArray);

	if (iRc == CCI_BUFFER_TOO_SMALL || iRc == 0)
	{
		buffer->pointer = (void *)malloc(iSize);
		buffer->size = iSize;
		iSize = cniElementByteArrayValue(&iRc, pElement, buffer);
		//pRetBuffer[iSize] = 0;
		if (iRc != 0)
		{
			free(buffer->pointer);
			buffer->pointer = NULL;
			buffer->size = 0;
		}
	}
	return(iSize);
}

///////////////////////////////////////////////////////////
// mqsixElementToString
char *mqsixElementToString(CciElement *pElement)
{
	CciElementType	iElementType;
	CciValueType	iElementValueType;
	int				iRc;
	static char		text[1024];
	char			strValue[512];
	CciChar			name[256];
	int				i;
	char			twoHexChar[3];
	
	CciSize			iSize;
	CciBool			valueBool;
	CciInt			valueInt;
	CciReal			valueReal;
	CciChar			*valueChar;
	struct CciTime	valueTime;
	struct CciTime	valueGmtTime;
	struct CciDate	valueDate;
	struct CciTimestamp valueTimestamp;
	struct CciTimestamp valueGmtTimestamp;
	struct CciByteArray valueByteArray;


	iElementType = cniElementType(&iRc, pElement) & CCI_ELEMENT_TYPE_MASK_GENERIC;

	switch(iElementType)
	{
	case CCI_ELEMENT_TYPE_UNKNOWN:
		strcpy(text, "<** Unknown **>");
		break;
	case CCI_ELEMENT_TYPE_VALUE:
		strcpy(text, "= ");
		break;
	case CCI_ELEMENT_TYPE_NAME:
		cniElementName(&iRc, pElement, name, sizeof(name));
		sprintf(text, "<%S>", name);
		break;
	case CCI_ELEMENT_TYPE_NAME_VALUE:
		cniElementName(&iRc, pElement, name, sizeof(name));
		sprintf(text, "<%S> = ", name);
		break;
	}

	switch(iElementType)
	{
	case CCI_ELEMENT_TYPE_UNKNOWN:
	case CCI_ELEMENT_TYPE_NAME:
		break;
	case CCI_ELEMENT_TYPE_VALUE:
	case CCI_ELEMENT_TYPE_NAME_VALUE:
		iElementValueType = cniElementValueType(&iRc, pElement);
		switch(iElementValueType)
		{
		case CCI_VALUE_TYPE_UNKNOWN:
			strcat(text, "<** Unknown **>");
			break;
		case CCI_VALUE_TYPE_BOOL:
			valueBool = cniElementBooleanValue(&iRc, pElement);
			if (valueBool == 0)
			{
				strcat(text, "false");
			}
			else
			{
				strcat(text, "true");
			}
			break;
		case CCI_VALUE_TYPE_INTEGER:
			valueInt = cniElementIntegerValue(&iRc, pElement);
/*
			_i64toa(valueInt, strValue, 10);
*/
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_REAL:
			valueReal = cniElementRealValue(&iRc, pElement);
			sprintf(strValue, "%f");
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_DECIMAL:
			// TBD
			break;
		case CCI_VALUE_TYPE_CHAR:
			// TBD
			iSize = mqsixElementCharValue(pElement, &valueChar);
			if (iSize == 0)
			{
				sprintf(strValue, "(Null)");
			}
			else
			{
				sprintf(strValue, "\"%S\"", valueChar);
				free(valueChar);
			}
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_TIME:
			valueTime = cniElementTimeValue(&iRc, pElement);
			sprintf(strValue, "TIME '%.2d:%.2d:%2.2f'",
				valueTime.hour, valueTime.minute, valueTime.second);
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_GMTTIME:
			valueGmtTime = cniElementGmtTimeValue(&iRc, pElement);
			sprintf(strValue, "GMTTIME '%.2d:%.2d:%2.2f'",
				valueGmtTime.hour, valueGmtTime.minute, valueGmtTime.second);
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_DATE:
			valueDate = cniElementDateValue(&iRc, pElement);
			sprintf(strValue, "DATE '%.4d-%.2d-%.2d'",
				valueDate.year, valueDate.month, valueDate.day);
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_TIMESTAMP:
			valueTimestamp = cniElementTimestampValue(&iRc, pElement);
			sprintf(strValue, "TIMESTAMP '%.4d-%.2d-%.2d %.2d:%.2d:%2.2f'",
				valueTimestamp.year, valueTimestamp.month, valueTimestamp.day,
				valueTimestamp.hour, valueTimestamp.minute, valueTimestamp.second);
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_GMTTIMESTAMP:
			valueGmtTimestamp = cniElementGmtTimestampValue(&iRc, pElement);
			sprintf(strValue, "GMTTIMESTAMP '%.4d-%.2d-%.2d %.2d:%.2d:%2.2f'",
				valueGmtTimestamp.year, valueGmtTimestamp.month, valueGmtTimestamp.day,
				valueGmtTimestamp.hour, valueGmtTimestamp.minute, valueGmtTimestamp.second);
			strcat(text, strValue);
			break;
		case CCI_VALUE_TYPE_BYTEARRAY:
			iSize = mqsixElementByteArrayValue(pElement, &valueByteArray);
			if (iSize == 0)
			{
				strcat(text, "(Null)");
			}
			else
			{
				strcat(text, "x'");
				for (i=0; i<valueByteArray.size; i++)
				{
					sprintf(twoHexChar, "%.2x", ((unsigned char *)valueByteArray.pointer)[i]);
					strcat(text, twoHexChar);
				}
				strcat(text, "'");
				free(valueByteArray.pointer);
			}
			break;
		case CCI_VALUE_TYPE_BITARRAY:
			break;
		}
		break;
	}
	return(text);
}

///////////////////////////////////////////////////////////
// mqsixWalkTree
void mqsixWalkTree(CciElement *pElement)
{
	char		*text;
	CciElement	*child;
	int			iRc;

	text = mqsixElementToString(pElement);
	mqsixLog("%s\n", text);
	child = cniFirstChild(&iRc, pElement);
	while(child != NULL)
	{
		mqsixWalkTree(child);
		child = cniNextSibling(&iRc, child);
	}
	return;
}

///////////////////////////////////////////////////////////
// mqsixLogInfo
void mqsixLogInfo(CciNode *pNode)
{
	int	iRc;
	CNI_BROKER_INFO_ST	brokerInfo;

	brokerInfo.brokerName.buffer = (CciChar *)malloc(sizeof(CciChar) * 100);
	brokerInfo.brokerName.bufferLength = 100;
	brokerInfo.executionGroupName.buffer = (CciChar *)malloc(sizeof(CciChar) * 100);
	brokerInfo.executionGroupName.bufferLength = 100;
	brokerInfo.messageFlowName.buffer = (CciChar *)malloc(sizeof(CciChar) * 100);
	brokerInfo.messageFlowName.bufferLength = 100;
	brokerInfo.queueManagerName.buffer = (CciChar *)malloc(sizeof(CciChar) * 100);
	brokerInfo.queueManagerName.bufferLength = 100;
	cniGetBrokerInfo(&iRc, pNode, &brokerInfo);
	mqsixLog("Version Id: %x\nBroker Name: %S\nExecution Group: %S\nMessage Flow: %S\nQueue Manager: %S\n",
		brokerInfo.versionId,
		brokerInfo.brokerName.buffer,
		brokerInfo.executionGroupName.buffer,
		brokerInfo.messageFlowName.buffer,
		brokerInfo.queueManagerName.buffer);
	free(brokerInfo.brokerName.buffer);
	free(brokerInfo.executionGroupName.buffer);
	free(brokerInfo.messageFlowName.buffer);
	free(brokerInfo.queueManagerName.buffer);
}

///////////////////////////////////////////////////////////
// mqsixLogTreeWithIndent
static void mqsixLogTreeWithIndent(CciElement *pElement, int iIndent)
{
	char		indentString[80];
	char		*text;
	int			iRc;
	int			i;
	CciElement	*child;

	text = mqsixElementToString(pElement);
	strcpy(indentString, "");
	for (i=0; i<iIndent; i++)
	{
		strcat(indentString, "  ");
	}
	mqsixLog("%s%s\n", indentString, text);

	child = cniFirstChild(&iRc, pElement);
	while(child != NULL)
	{
		mqsixLogTreeWithIndent(child, iIndent+1);
		child = cniNextSibling(&iRc, child);
	}
}

///////////////////////////////////////////////////////////
// mqsixLogTree
void mqsixLogTree(CciElement *pElement)
{
	mqsixLogTreeWithIndent(pElement, 0);
	return;
}

///////////////////////////////////////////////////////////
// mqsixDebugBreak
void mqsixDebugBreak()
{
#ifdef _WIN32
	if (IsDebuggerPresent())
	{
		DebugBreak();
	}
#endif
}

///////////////////////////////////////////////////////////
// mqsixASCIIToUnicode
CciChar* mqsixASCIIToUnicode(const char* aStr)
{
	int	iLength = sizeof(CciChar) * (strlen(aStr) + 1);
	CciChar	*ucsString = (CciChar *)malloc(iLength);
	int	iRc;
	cciMbsToUcs(
		&iRc,
		aStr,
		ucsString,
		iLength,
		1208);
	return(ucsString);
}

///////////////////////////////////////////////////////////
// mqsixUnicodeToASCII
char *mqsixUnicodeToASCII(CciChar* ucsString)
{
	int	iRc;
	int iLength = mqsixStringLengthCharacters(ucsString) + 1;
	char *mbString = (char *)malloc(iLength);
	cciUcsToMbs(
		&iRc,
		ucsString,
		mbString,
		iLength,
		1208);
	return(mbString);
}

///////////////////////////////////////////////////////////
// mqsixFindPart
// Search the string for '.' separated parts.
// On return, the function points past the last '.' or NULL if there was no '.'
static char *mqsixFindPart(char *path, char *part)
{
	char	*ptr;
	int		iSize;
	ptr = strchr(path, '.');
	if (ptr == 0)
	{
		strcpy(part, path);
	}
	else
	{
		iSize = ptr-path;
		memcpy(part, path, iSize);
		part[iSize] = '\0';
		ptr++;
	}
	return(ptr);
}


///////////////////////////////////////////////////////////
// mqsixFindElement
CciElement *mqsixFindElement(char *path, CciElement *pElement)
{
	/*
	Find element 
	Get the part (part is current ptr to first '.' or '\0')
	skip this one because is should be root
	Get the part
	is there a child of current element with "part" name?
	No - return false
	Yes - do we have another part
	   - no - return the element
	   - yes - loop again
	*/
	char	part[100];
	int		iRc;
	CciChar	name[100];
	char	*asciiName;
	int		bFound;

	mqsixDebugBreak();
	mqsixLog(">> Looking for element: %s\n", path);
	path = mqsixFindPart(path, part);
	cniElementName(&iRc, pElement, name, sizeof(name));
	asciiName = mqsixUnicodeToASCII(name);
	iRc = strcmp(asciiName, part);
	free(asciiName);
	if (iRc != 0)
	{
		// root parts don't match
		return(NULL);
	}

	while(path != NULL)
	{
		path = mqsixFindPart(path, part);
		pElement = cniFirstChild(&iRc, pElement);
		bFound = 0;
		while(pElement != NULL && !bFound)
		{
			cniElementName(&iRc, pElement, name, sizeof(name));
			asciiName = mqsixUnicodeToASCII(name);
			iRc = strcmp(asciiName, part);
			free(asciiName);
			if (iRc == 0)
			{
				bFound = 1;
			}
			else
			{
				pElement = cniNextSibling(&iRc, pElement);
			}
		}
		if (!bFound)
		{
			return(NULL);
		}
	}
	return(pElement);
}

///////////////////////////////////////////////////////////
// mqsixFindChild
CciElement *mqsixFindChild(char *childName, CciElement *pElement)
{
	int		iRc;
	CciChar	*uChildName;
	uChildName = mqsixASCIIToUnicode(childName);
	pElement = cniSearchFirstChild(&iRc, pElement, CCI_COMPARE_MODE_NAME, 0, uChildName);
	free(uChildName);
	return(pElement);
}

///////////////////////////////////////////////////////////
// mqsixNewElement
CciElement *mqsixNewElement(char *path, CciMessage *pMessage)
{
	char		part[100];
	CciElement	*pElement;
	CciElement	*pElement2;
	CciChar		*tempName;
	int			iRc;

	path = mqsixFindPart(path, part);
	if (strcmp(part, "Root") != 0)
	{
		mqsixLog(">> mqsixNewElement: Must start with Root.\n");
		return(NULL);
	}

	pElement = cniRootElement(&iRc, pMessage);
	while (path != NULL)
	{
		path = mqsixFindPart(path, part);
		pElement2 = mqsixFindChild(part, pElement);
		if (pElement2 == NULL)
		{
			// Add "part" as a child of pElement
			pElement2 = cniCreateElementAsLastChild(&iRc, pElement);
			tempName = mqsixASCIIToUnicode(part);
			cniSetElementName(&iRc, pElement2, tempName);
			free(tempName);
			cniSetElementType(&iRc, pElement2, CCI_ELEMENT_TYPE_NAME);
		}
		pElement = pElement2;
	}
	return(pElement);
}


///////////////////////////////////////////////////////////
// mqsixBrokerInfo
void mqsixBrokerInfo(CciNode *nodeObject, MQSIX_NODE_INFO *pNodeInfo)
{
	CNI_BROKER_INFO_ST	brokerInfo;
	int	iRc;
	brokerInfo.brokerName.buffer = pNodeInfo->brokerName;
	brokerInfo.brokerName.bufferLength = MQSIX_MAX_STRING;
	brokerInfo.executionGroupName.buffer = pNodeInfo->executionGroupName;
	brokerInfo.executionGroupName.bufferLength = MQSIX_MAX_STRING;
	brokerInfo.messageFlowName.buffer = pNodeInfo->messageFlowName;
	brokerInfo.messageFlowName.bufferLength = MQSIX_MAX_STRING;
	brokerInfo.queueManagerName.buffer = pNodeInfo->queueManagerName;
	brokerInfo.queueManagerName.bufferLength = MQSIX_MAX_STRING;
	cniGetBrokerInfo(&iRc, nodeObject, &brokerInfo);
}

///////////////////////////////////////////////////////////
// mqsixRegisterMessages
#ifdef _WIN32
void mqsixRegisterMessages(HMODULE hModule, char *messageSet)
{
	char	strFileName[512];
	char	strRegistryPath[512];
	HKEY	key;
	DWORD	iDisposition;
	DWORD	iValue;
	DWORD	iRc;

	GetModuleFileName(hModule, strFileName, sizeof(strFileName));
	mqsixLog("File name of DLL: %s\n", strFileName);
	sprintf(strRegistryPath,
		"SYSTEM\\CurrentControlSet\\Services\\Eventlog\\Application\\%s", messageSet);
	
	iRc = RegCreateKeyEx(HKEY_LOCAL_MACHINE,
		strRegistryPath,
		0,
		"",
		REG_OPTION_NON_VOLATILE,
		KEY_WRITE,
		NULL,
		&key,
		&iDisposition);
	if (iRc != ERROR_SUCCESS)
	{
		mqsixLog("Failed to RegCreateKeyEx: %d\n", iRc);
		return;
	}
	if (iDisposition == REG_CREATED_NEW_KEY)
	{
		RegSetValueEx(key, "EventMessageFile", 0, REG_SZ, (PBYTE)strFileName, strlen(strFileName) + 1);
		iValue = 7;
		RegSetValueEx(key, "TypesSupported", 0, REG_DWORD, (PBYTE)&iValue, sizeof(iValue));
	}
	RegCloseKey(key);
}
#endif
